@@ -1,19 +1,5 @@
 """Code common to Connection and Channel objects."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>)
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 from __future__ import absolute_import, unicode_literals
 
 from vine import ensure_promise, promise
@@ -26,14 +12,17 @@ __all__ = ['AbstractChannel']
 
 
 class AbstractChannel(object):
-    """Superclass for both the Connection, which is treated
-    as channel 0, and other user-created Channel objects.
+    """Superclass for Connection and Channel.
+
+    The connection is treated as channel 0, then comes
+    user-created channel objects.
 
     The subclasses must have a _METHOD_MAP class property, mapping
     between AMQP method signatures and Python methods.
-
     """
+
     def __init__(self, connection, channel_id):
+        self.is_closing = False
         self.connection = connection
         self.channel_id = channel_id
         connection.channels[channel_id] = self
@@ -72,7 +61,7 @@ class AbstractChannel(object):
         return p
 
     def close(self):
-        """Close this Channel or Connection"""
+        """Close this Channel or Connection."""
         raise NotImplementedError('Must be overriden in subclass')
 
     def wait(self, method, callback=None, timeout=None, returns_tuple=False):
@@ -92,6 +81,7 @@ class AbstractChannel(object):
 
             if p.value:
                 args, kwargs = p.value
+                args = args[1:]  # We are not returning method back
                 return args if returns_tuple else (args and args[0])
         finally:
             for i, m in enumerate(method):
@@ -118,17 +108,13 @@ class AbstractChannel(object):
         try:
             listeners = [self._callbacks[method_sig]]
         except KeyError:
-            listeners = None
+            listeners = []
+        one_shot = None
         try:
             one_shot = self._pending.pop(method_sig)
         except KeyError:
             if not listeners:
                 return
-        else:
-            if listeners is None:
-                listeners = [one_shot]
-            else:
-                listeners.append(one_shot)
 
         args = []
         if amqp_method.args:
@@ -139,6 +125,9 @@ class AbstractChannel(object):
         for listener in listeners:
             listener(*args)
 
+        if one_shot:
+            one_shot(method_sig, *args)
+
     #: Placeholder, the concrete implementations will have to
     #: supply their own versions of _METHOD_MAP
     _METHODS = {}
